import requests
from yarl import URL
from .base import Backend

class TelegramBackend(Backend):

    def __init__(self, apikey, chat_id):
        self.apikey = apikey
        self.chat_id = chat_id

    def notify(self, message):
        url = URL('https://api.telegram.org') / f'bot{self.apikey}' / 'sendMessage'
        url = url.with_query(chat_id=self.chat_id, text=message)
        requests.get(url).raise_for_status()



import os
import pytest

import woof.backends.base


@pytest.fixture
def custom_backend_class(request):
    class CustomBackend(woof.backends.base.Backend):

        messages = []

        def __init__(self, param1, param2):
            self.param1 = param1
            self.param2 = param2

        def notify(self, message):
            self.messages.append({'param1': self.param1, 'param2': self.param2, 'message': message})

    @request.addfinalizer
    def cleanup(): # pylint: disable=unused-variable
        woof.backends.base.BACKENDS.remove(CustomBackend)

    return CustomBackend

@pytest.fixture
def environ_set(request):

    def set(key, value):
        prev = os.environ.get(key)
        os.environ[key] = value
        @request.addfinalizer
        def cleanup(): # pylint: disable=unused-variable
            if prev is None:
                os.environ.pop(key, None)
            else:
                os.environ[key] = prev
    return set



from .backends import telegram_backend # pylint: disable=unused-import

from .backends.base import BACKENDS


def notify(message):
    """Sends the specified message through available backends
    """
    for backend_class in BACKENDS:
        backend = backend_class.try_configure()
        if backend is not None:
            backend.notify(message)
